@@ -82,8 +82,6 @@ class Prologue(object):
         # Create a store for directives
         self.directives = {}
         if register_prime: register_prime_directives(self)
-        # Setup blank loop for converting from output line to input file and line
-        self.lookup = None
 
     # ==========================================================================
     # Property Setters/Getters
@@ -177,23 +175,36 @@ class Prologue(object):
     # Registry Passthroughs
     # ==========================================================================
 
-    def add_file(self, path):
+    def add_file(self, path, ignore_duplicate=False):
         """ Add a file to the registry.
 
         Args:
-            path: Path to add
+            path            : Path to add
+            ignore_duplicate: Don't raise an error if the file or folder already
+                              exists in the registry (default: False)
         """
-        self.registry.add_file(path)
+        self.registry.add_file(path, ignore_duplicate=ignore_duplicate)
 
-    def add_folder(self, path, search_for=None, recursive=False):
+    def add_folder(
+        self,
+        path,
+        search_for      =None,
+        recursive       =False,
+        ignore_duplicate=False,
+    ):
         """ Add a folder to the registry, see Registry.add_folder for more info.
 
         Args:
-            path      : Path to the root folder to add
-            search_for: Provide a file extension to search for
-            recursive : Whether to search recursively in this folder
+            path            : Path to the root folder to add
+            search_for      : Provide a file extension to search for
+            recursive       : Whether to search recursively in this folder
+            ignore_duplicate: Don't raise an error if the file or folder already
+                              exists in the registry (default: False)
         """
-        self.registry.add_folder(path, search_for=search_for, recursive=recursive)
+        self.registry.add_folder(
+            path, search_for=search_for, recursive=recursive,
+            ignore_duplicate=ignore_duplicate,
+        )
 
     # ==========================================================================
     # Directives
@@ -267,7 +278,9 @@ class Prologue(object):
     # Evaluation
     # ==========================================================================
 
-    def evaluate(self, filename, defines=None, callback=None):
+    def evaluate(
+        self, filename, defines=None, callback=None, lookup=None, included=None,
+    ):
         """ Iterable evaluation function which returns fully preprocessed stream
 
         Args:
@@ -275,6 +288,10 @@ class Prologue(object):
             defines : Predefined values to initialise the context (default: None)
             callback: Optional callback provided to all directives to expose
                       state during parse to an external routine (default: None)
+            lookup  : Optionally provide an empty list to capture lookup for
+                      resolving output line to input line and file (default: None)
+            included: Optionally provide an empty list to capture all files that
+                      were included during preprocessing (default: None)
 
         Yields: Stream of fully preprocessed lines
         """
@@ -287,11 +304,12 @@ class Prologue(object):
             initial_state =defines,
         )
         # Use inner evaluation routine to get each line one at a time
-        self.lookup = []
         for line in self.evaluate_inner(filename, context, callback=callback):
             final = context.substitute(line)
-            self.lookup.append((final.file, final.number))
+            if isinstance(lookup, list): lookup.append((final.file, final.number))
             yield str(final)
+        # Get all files included as part of the trace
+        if isinstance(included, list): included += list(set(context.trace))
 
     def evaluate_inner(self, filename, context, callback=None):
         """
@@ -447,12 +465,13 @@ class Prologue(object):
     # Lookup
     # ==========================================================================
 
-    def resolve(self, line, before=2, after=2):
+    def resolve(self, lookup, line, before=2, after=2):
         """
         Use the line lookup to resolve which input file and line number produced
         each line of the output.
 
         Args:
+            lookup: A lookup list generated by Prologue during evaluation
             line  : Line number (from 1 to number of output lines, not an index)
             before: How many lines to include in the snippet before the target
             after : How many lines to include in the snippet after the target
@@ -460,17 +479,15 @@ class Prologue(object):
         Returns: Tuple of input RegistryFile and the line number
         """
         # Sanity checks
-        if not self.lookup:
-            raise PrologueError(
-                "Lookup does not yet exist - have you called 'evaluate'?"
-            )
+        if not lookup or not isinstance(lookup, list):
+            raise PrologueError(f"Lookup is either empty or not a list: {lookup}")
         elif not isinstance(line, int):
             raise PrologueError(f"Line number must be an integer - not '{line}'")
-        elif line < 1 or line > len(self.lookup):
+        elif line < 1 or line > len(lookup):
             raise PrologueError(
-                f"Line {line} is out of valid range 1-{len(self.lookup)}"
+                f"Line {line} is out of valid range 1-{len(lookup)}"
             )
         # Use lookup to get the file and line number
-        r_file, line_no = self.lookup[line-1]
+        r_file, line_no = lookup[line-1]
         # Return the input file and line number
         return r_file, line_no, r_file.snippet(line_no, before, after)
